import matplotlib.pyplot as plt
import numpy as np
from scipy.interpolate import interp1d
from scipy.integrate   import odeint

def derotate( v, cosphi, sinphi):
    m = np.array([[cosphi,-sinphi],[sinphi,cosphi]])
    return np.dot(m,v)

def dudt(u,t,axi,ayi):
    dudt = np.empty(4)
    # u = (x,y,vx,vy) # differential equation for phi?

    a = np.array([axi(t),ayi(t)])

    # de-rotate acceleration vector
    # check magnitude of velocity
    v = np.sqrt(u[2]**2+u[3]**2)
    if v > 0.01:
        # de-rotate acceleration vector
        ax, ay = derotate(a, u[2]/v, u[3]/v)
    else:
        # assume the car goes in x-direction      
        ax, ay = a

    dudt[0] = u[2]   # dxdt  = vx
    dudt[1] = u[3]   # dydt  = vy
    dudt[2] = ax     # dvxdt = ax
    dudt[3] = ay     # dvydt = ax
    return dudt

# change of orientation of coordinate system:
# start in positive x-direction
t, ay, ax, az = np.loadtxt('../../hans/data/autofahrt.txt', 
                            skiprows=2,unpack=True)
ax = -ax
ay = -ay

# Generate interpolation objects for ax, ay:
axi = interp1d(t,ax,kind=1, bounds_error=False, fill_value=0.0)
ayi = interp1d(t,ay,kind=1, bounds_error=False, fill_value=0.0)

time = np.linspace(0.0,max(t),1000)
#time = np.linspace(max(t),0.0,1000) ?
u0   = np.array([0.0,0.0,0.0,0.0]) # initial condition
u    = odeint(dudt, u0, time, args=(axi,ayi))

plt.axis('equal')
plt.plot(u[:,0],u[:,1])
